package com.kdanmobile.reader

import android.annotation.SuppressLint
import android.arch.lifecycle.ViewModel
import android.os.Handler
import android.os.Message
import android.util.Log
import com.kdanmobile.kmpdfkit.manager.KMPDFFactory
import com.kdanmobile.kmpdfkit.manager.controller.KMPDFDocumentController
import java.lang.ref.PhantomReference
import java.lang.ref.ReferenceQueue
import java.util.*

class ReaderModel {
    private var filename: String? = null
    var password: String = ""
        private set

    var kmpdfFactory: KMPDFFactory? = null
        private set

    @SuppressLint("StaticFieldLeak")
    var kmpdfDocumentController: KMPDFDocumentController? = null
        private set

    var onPdfChangedListener: OnPdfChangedListener? = null

    private val queue = ReferenceQueue<KMPDFFactory>()
    private var pr: PhantomReference<KMPDFFactory>? = null

    var referenceCount = 0

    @Synchronized
    fun initKMPDFFactory(kmpdfFactory: KMPDFFactory?, filename: String?, password: String = "") {

        this.kmpdfFactory?.run {
            destroy()
        }

        this.kmpdfFactory = kmpdfFactory
        this.filename = filename
        this.password = password
        this.kmpdfFactory?.also {
            pr = PhantomReference(it, queue)
        }
    }

    fun initKMPDFDocumentController() {
        kmpdfDocumentController = kmpdfFactory?.getController(KMPDFFactory.ControllerType.DOCUMENT) as KMPDFDocumentController
    }

    @Synchronized
    fun destroy() {
        filename = null
        password = ""
        kmpdfFactory?.destroy()
        kmpdfFactory = null
        kmpdfDocumentController = null
    }

    fun register() {
        referenceCount++
    }

    fun unregister() {
        referenceCount--
        if (referenceCount == 0) {
            destroy()
        }
    }
}